@@ -192,9 +192,9 @@ nautilus_directory_get_file_metadata (NautilusDirectory *directory,
 	const char       *non_null_default;
 	CORBA_char       *corba_value;
 
-	g_return_val_if_fail (NAUTILUS_IS_DIRECTORY (directory), NULL);
-	g_return_val_if_fail (!eel_str_is_empty (file_name), NULL);
-	g_return_val_if_fail (!eel_str_is_empty (key), NULL);
+	g_return_val_if_fail (NAUTILUS_IS_DIRECTORY (directory), g_strdup (default_metadata));
+	g_return_val_if_fail (!eel_str_is_empty (file_name), g_strdup (default_metadata));
+	g_return_val_if_fail (!eel_str_is_empty (key), g_strdup (default_metadata));
 	
 	/* We can't pass NULL as a CORBA_string - pass "" instead. */
 	non_null_default = default_metadata != NULL ? default_metadata : "";
@@ -203,7 +203,12 @@ nautilus_directory_get_file_metadata (NautilusDirectory *directory,
 
 	corba_value = Nautilus_Metafile_get (get_metafile (directory), file_name, key, non_null_default, &ev);
 
-	/* FIXME bugzilla.gnome.org 46664: examine ev for errors */
+	if (ev._major != CORBA_NO_EXCEPTION) {
+		g_warning ("Failed to get file metadata.");
+		CORBA_exception_free (&ev);
+		return g_strdup (default_metadata);
+	}
+
 	CORBA_exception_free (&ev);
 
 	if (eel_str_is_empty (corba_value)) {
