@@ -38,6 +38,7 @@
 #include <eel/eel-stock-dialogs.h>
 #include <eel/eel-vfs-extensions.h>
 
+#include <glib/gstdio.h>
 #include <gnome.h>
 #include <gdk/gdkdnd.h>
 #include <gtk/gtklabel.h>
@@ -1858,8 +1859,15 @@ nautilus_file_operations_copy_move (const GList *item_uris,
 		target_uri = NULL;
 		if (target_dir != NULL) {
 			if (target_is_trash) {
-				gnome_vfs_find_directory (source_uri, GNOME_VFS_DIRECTORY_KIND_TRASH,
-							  &target_dir_uri, FALSE, FALSE, 0777);			
+				result = gnome_vfs_find_directory (source_uri, GNOME_VFS_DIRECTORY_KIND_TRASH,
+								   &target_dir_uri, FALSE, FALSE, 0777);
+				if (result == GNOME_VFS_ERROR_NOT_FOUND && source_dir_uri != NULL) {
+					/* source_uri may be a broken symlink */
+					result = gnome_vfs_find_directory (source_dir_uri, GNOME_VFS_DIRECTORY_KIND_TRASH,
+									   &target_dir_uri, FALSE, FALSE, 0777);
+				}
+
+				result = GNOME_VFS_OK;
 			}
 			if (target_dir_uri != NULL) {
 				target_uri = append_basename (target_dir_uri, source_uri);
@@ -2386,7 +2394,6 @@ nautilus_file_operations_new_file_from_template (GtkWidget *parent_view,
 						 const char *parent_dir,
 						 const char *target_filename,
 						 const char *template_uri,
-						 gboolean move_template,
 						 NautilusNewFileCallback done_callback,
 						 gpointer data)
 {
@@ -2435,9 +2442,6 @@ nautilus_file_operations_new_file_from_template (GtkWidget *parent_view,
 	sync_transfer_info->debuting_uris = state->debuting_uris;
 
 	options = GNOME_VFS_XFER_USE_UNIQUE_NAMES;
-	if (move_template) {
-		options |= GNOME_VFS_XFER_REMOVESOURCE;
-	}
 
 	gnome_vfs_async_xfer (&state->handle, source_uri_list, target_uri_list,
 	      		      options,
@@ -2452,6 +2456,30 @@ nautilus_file_operations_new_file_from_template (GtkWidget *parent_view,
 	gnome_vfs_uri_unref (parent_uri);
 }
 
+struct NewFileData {
+	char *tmp_file;
+	NautilusNewFileCallback done_callback;
+	gpointer callback_data;
+};
+
+static void
+new_file_from_temp_callback (const char *new_file_uri,
+			     gpointer    callback_data)
+{
+	struct NewFileData *data = callback_data;
+
+	/* Remove the template file
+	 * Gnome-vfs can do this, but it caused problem, see bug #309592
+	 */
+	g_remove (data->tmp_file);
+	g_free (data->tmp_file);
+	
+	(data->done_callback) (new_file_uri, data->callback_data);
+
+	g_free (data);
+}
+
+
 void 
 nautilus_file_operations_new_file (GtkWidget *parent_view, 
 				   const char *parent_dir,
@@ -2459,6 +2487,7 @@ nautilus_file_operations_new_file (GtkWidget *parent_view,
 				   NautilusNewFileCallback done_callback,
 				   gpointer data)
 {
+	struct NewFileData *new_data;
 	char source_file_str[] = "/tmp/nautilus-sourceXXXXXX";
 	char *source_file_uri;
 	FILE *source_file;
@@ -2484,13 +2513,17 @@ nautilus_file_operations_new_file (GtkWidget *parent_view,
 
 	source_file_uri = gnome_vfs_get_uri_from_local_path (source_file_str);
 
+	new_data = g_new (struct NewFileData, 1);
+	new_data->tmp_file = g_strdup (source_file_str);
+	new_data->done_callback = done_callback;
+	new_data->callback_data = data;
+	
 	nautilus_file_operations_new_file_from_template (parent_view, 
 							 parent_dir,
 							 target_filename,
 							 source_file_uri,
-							 TRUE,
-							 done_callback,
-							 data);
+							 new_file_from_temp_callback,
+							 new_data);
 
 	g_free (source_file_uri);
 	g_free (target_filename);
@@ -2686,6 +2719,8 @@ nautilus_file_operations_empty_trash (GtkWidget *parent_view)
 void
 nautilus_self_check_file_operations (void)
 {
+	setlocale (LC_MESSAGES, "C");
+
 	/* test the next duplicate name generator */
 	EEL_CHECK_STRING_RESULT (get_duplicate_name (" (copy)", 1), " (another copy)");
 	EEL_CHECK_STRING_RESULT (get_duplicate_name ("foo", 1), "foo (copy)");
@@ -2728,6 +2763,8 @@ nautilus_self_check_file_operations (void)
 	EEL_CHECK_STRING_RESULT (get_duplicate_name ("foo (122nd copy).txt", 1), "foo (123rd copy).txt");
 	EEL_CHECK_STRING_RESULT (get_duplicate_name ("foo (123rd copy)", 1), "foo (124th copy)");
 	EEL_CHECK_STRING_RESULT (get_duplicate_name ("foo (123rd copy).txt", 1), "foo (124th copy).txt");
+
+	setlocale (LC_MESSAGES, "");
 }
 
 #endif
