@@ -44,7 +44,6 @@ typedef enum {
 } Knowledge;
 
 typedef struct {
-	int automatic_emblems_as_integer;
 	char emblem_keywords[1];
 } NautilusFileSortByEmblemCache;
 
@@ -66,8 +65,6 @@ struct NautilusFileDetails
 	GnomeVFSFileInfo *info;
 	GnomeVFSResult get_info_error;
 
-	NautilusMonitor *monitor;
-	
 	guint directory_count;
 
 	guint deep_directory_count;
