@@ -467,7 +467,10 @@ nautilus_icon_canvas_item_get_image (NautilusIconCanvasItem *item,
 	int width, height;
 	int item_offset_x, item_offset_y;
 	ArtIRect icon_rect;
+	ArtIRect emblem_rect;
 	GdkPixbuf *pixbuf;
+	GdkPixbuf *emblem_pixbuf;
+	EmblemLayout emblem_layout;
 	double item_x, item_y;
 	
 	g_return_val_if_fail (NAUTILUS_IS_ICON_CANVAS_ITEM (item), NULL);
@@ -496,14 +499,42 @@ nautilus_icon_canvas_item_get_image (NautilusIconCanvasItem *item,
 				 gdk_colormap_get_visual (colormap)->depth);
 	gdk_drawable_set_colormap (GDK_DRAWABLE (pixmap), colormap);
 
-	pixbuf = item->details->pixbuf;
+	pixbuf = gdk_pixbuf_new (GDK_COLORSPACE_RGB,
+				 TRUE,
+				 gdk_pixbuf_get_bits_per_sample (item->details->pixbuf),
+				 width, height);
+	gdk_pixbuf_fill (pixbuf, 0x00000000);
+
+	gdk_pixbuf_composite (item->details->pixbuf, pixbuf,
+			      item_offset_x, item_offset_y,
+			      gdk_pixbuf_get_width (item->details->pixbuf),
+			      gdk_pixbuf_get_height (item->details->pixbuf),
+			      item_offset_x, item_offset_y, 1.0, 1.0,
+			      GDK_INTERP_BILINEAR, 255);
+
+	icon_rect.x0 = item_offset_x;
+	icon_rect.y0 = item_offset_y;
+	icon_rect.x1 = item_offset_x + gdk_pixbuf_get_width (item->details->pixbuf);
+	icon_rect.y1 = item_offset_y + gdk_pixbuf_get_height (item->details->pixbuf);
+
+	emblem_layout_reset (&emblem_layout, item, icon_rect);
+	while (emblem_layout_next (&emblem_layout, &emblem_pixbuf, &emblem_rect)) {
+		gdk_pixbuf_composite (emblem_pixbuf, pixbuf,
+				      emblem_rect.x0, emblem_rect.y0,
+				      gdk_pixbuf_get_width (emblem_pixbuf),
+				      gdk_pixbuf_get_height (emblem_pixbuf),
+				      emblem_rect.x0, emblem_rect.y0,
+				      1.0, 1.0,
+				      GDK_INTERP_BILINEAR, 255);
+	}
+
 	gc = gdk_gc_new (pixmap);
 	gdk_draw_rectangle (pixmap, GTK_WIDGET (canvas)->style->white_gc,
 			    TRUE,
 			    0, 0,
 			    width, height);
 	gdk_draw_pixbuf (pixmap, gc, pixbuf, 
-			 0, 0, item_offset_x, item_offset_y,
+			 0, 0, 0, 0,
 			 gdk_pixbuf_get_width (pixbuf), gdk_pixbuf_get_height (pixbuf),
 			 GDK_RGB_DITHER_NORMAL,
 			 0, 0);
@@ -520,20 +551,18 @@ nautilus_icon_canvas_item_get_image (NautilusIconCanvasItem *item,
 	g_object_unref (gc);
 	  
 	gdk_pixbuf_render_threshold_alpha (pixbuf, *mask,
-					   0, 0, item_offset_x, item_offset_y,
+					   0, 0, 0, 0,
 					   gdk_pixbuf_get_width (pixbuf), gdk_pixbuf_get_height (pixbuf),
 					   128);
 	
 	draw_embedded_text (item, GDK_DRAWABLE (pixmap),
 			    item_offset_x, item_offset_y);
 
-	icon_rect.x0 = item_offset_x;
-	icon_rect.y0 = item_offset_y;
-	icon_rect.x1 = item_offset_x + gdk_pixbuf_get_width (pixbuf);
-	icon_rect.y1 = item_offset_y + gdk_pixbuf_get_height (pixbuf);
 	draw_label_text (item, GDK_DRAWABLE (pixmap), FALSE, icon_rect);
 	draw_label_text (item, GDK_DRAWABLE (*mask), TRUE, icon_rect);
-	
+
+	gdk_pixbuf_unref (pixbuf);
+
 	return pixmap;
 }
 
@@ -1047,7 +1076,7 @@ draw_or_measure_label_text (NautilusIconCanvasItem *item,
 		if (needs_frame && !needs_highlight && details->text_width > 0 && details->text_height > 0) {
 			draw_frame (item, 
 				    drawable,
-				    eel_gdk_color_to_rgb (&GTK_WIDGET (container)->style->base[GTK_STATE_NORMAL]),
+				    container->details->normal_color_rgba,
 				    create_mask,
 				    text_rect.x0,
 				    text_rect.y0,
@@ -2893,32 +2922,32 @@ nautilus_icon_canvas_item_class_init (NautilusIconCanvasItemClass *class)
 		object_class,
 		PROP_EDITABLE_TEXT,
 		g_param_spec_string ("editable_text",
-				     _("editable text"),
-				     _("the editable label"),
+				     "editable text",
+				     "the editable label",
 				     "", G_PARAM_READWRITE));
 
         g_object_class_install_property (
 		object_class,
 		PROP_ADDITIONAL_TEXT,
 		g_param_spec_string ("additional_text",
-				     _("additional text"),
-				     _("some more text"),
+				     "additional text",
+				     "some more text",
 				     "", G_PARAM_READWRITE));
 
         g_object_class_install_property (
 		object_class,
 		PROP_HIGHLIGHTED_FOR_SELECTION,
 		g_param_spec_boolean ("highlighted_for_selection",
-				      _("highlighted for selection"),
-				      _("whether we are highlighted for a selection"),
+				      "highlighted for selection",
+				      "whether we are highlighted for a selection",
 				      FALSE, G_PARAM_READWRITE)); 
 
         g_object_class_install_property (
 		object_class,
 		PROP_HIGHLIGHTED_AS_KEYBOARD_FOCUS,
 		g_param_spec_boolean ("highlighted_as_keyboard_focus",
-				      _("highlighted as keyboard focus"),
-				      _("whether we are highlighted to render keyboard focus"),
+				      "highlighted as keyboard focus",
+				      "whether we are highlighted to render keyboard focus",
 				      FALSE, G_PARAM_READWRITE)); 
 
 
@@ -2926,8 +2955,8 @@ nautilus_icon_canvas_item_class_init (NautilusIconCanvasItemClass *class)
 		object_class,
 		PROP_HIGHLIGHTED_FOR_DROP,
 		g_param_spec_boolean ("highlighted_for_drop",
-				      _("highlighted for drop"),
-				      _("whether we are highlighted for a D&D drop"),
+				      "highlighted for drop",
+				      "whether we are highlighted for a D&D drop",
 				      FALSE, G_PARAM_READWRITE)); 
 
 	item_class->update = nautilus_icon_canvas_item_update;
