@@ -42,6 +42,8 @@
 
 #define AUTO_SCROLL_MARGIN 20
 
+#define HOVER_EXPAND_TIMEOUT 1000
+
 struct _NautilusTreeViewDragDestDetails {
 	GtkTreeView *tree_view;
 
@@ -54,12 +56,16 @@ struct _NautilusTreeViewDragDestDetails {
 
 	guint highlight_id;
 	guint scroll_id;
+	guint expand_id;
 };
 
 enum {
 	GET_ROOT_URI,
 	GET_FILE_FOR_PATH,
 	MOVE_COPY_ITEMS,
+	HANDLE_URL,
+	HANDLE_URI_LIST,
+	HANDLE_TEXT,
 	LAST_SIGNAL
 };
 
@@ -72,11 +78,11 @@ GNOME_CLASS_BOILERPLATE (NautilusTreeViewDragDest,
 			 nautilus_tree_view_drag_dest,
 			 GObject, G_TYPE_OBJECT);
 
-static GtkTargetEntry drag_types [] = {
+static const GtkTargetEntry drag_types [] = {
 	{ NAUTILUS_ICON_DND_GNOME_ICON_LIST_TYPE, 0, NAUTILUS_ICON_DND_GNOME_ICON_LIST },
 	{ NAUTILUS_ICON_DND_URI_LIST_TYPE, 0, NAUTILUS_ICON_DND_URI_LIST },
-	{ NAUTILUS_ICON_DND_URL_TYPE, 0, NAUTILUS_ICON_DND_URL }
-	/* FIXME: Should handle emblems once the list view supports them */
+	{ NAUTILUS_ICON_DND_URL_TYPE, 0, NAUTILUS_ICON_DND_URL },
+	{ NAUTILUS_ICON_DND_KEYWORD_TYPE, 0, NAUTILUS_ICON_DND_KEYWORD }
 };
 
 
@@ -126,11 +132,38 @@ static void
 remove_scroll_timeout (NautilusTreeViewDragDest *dest)
 {
 	if (dest->details->scroll_id) {
-		gtk_timeout_remove (dest->details->scroll_id);
+		g_source_remove (dest->details->scroll_id);
 		dest->details->scroll_id = 0;
 	}
 }
 
+static int
+expand_timeout (gpointer data)
+{
+	GtkTreeView *tree_view;
+	GtkTreePath *drop_path;
+	
+	tree_view = GTK_TREE_VIEW (data);
+	
+	gtk_tree_view_get_drag_dest_row (tree_view, &drop_path, NULL);
+	
+	if (drop_path) {
+		gtk_tree_view_expand_row (tree_view, drop_path, FALSE);
+		gtk_tree_path_free (drop_path);
+	}
+
+	return FALSE;
+}
+
+static void
+remove_expand_timeout (NautilusTreeViewDragDest *dest)
+{
+	if (dest->details->expand_id) {
+		g_source_remove (dest->details->expand_id);
+		dest->details->expand_id = 0;
+	}
+}
+
 static gboolean
 highlight_expose (GtkWidget *widget,
 		  GdkEventExpose *event,
@@ -254,8 +287,11 @@ file_for_path (NautilusTreeViewDragDest *dest, GtkTreePath *path)
 		g_signal_emit (dest, signals[GET_FILE_FOR_PATH], 0, path, &file);
 	} else {
 		uri = get_root_uri (dest);
-		
-		file = nautilus_file_get (uri);
+
+		file = NULL;
+		if (uri != NULL) {
+			file = nautilus_file_get (uri);
+		}
 		
 		g_free (uri);
 	}
@@ -302,6 +338,10 @@ get_drop_target (NautilusTreeViewDragDest *dest,
 	char *target;
 
 	file = file_for_path (dest, path);
+	if (file == NULL) {
+		return NULL;
+	}
+	
 	target = nautilus_file_get_drop_target_uri (file);
 	nautilus_file_unref (file);
 	
@@ -316,7 +356,9 @@ get_drop_action (NautilusTreeViewDragDest *dest,
 	char *drop_target;
 	guint action;
 	
-	if (!dest->details->have_drag_data || !dest->details->drag_list) {
+	if (!dest->details->have_drag_data ||
+	    (dest->details->drag_type == NAUTILUS_ICON_DND_GNOME_ICON_LIST &&
+	     dest->details->drag_list == NULL)) {
 		return 0;
 	}
 
@@ -337,9 +379,23 @@ get_drop_action (NautilusTreeViewDragDest *dest,
 		g_free (drop_target);
 		
 		return action;
+		
+	case NAUTILUS_ICON_DND_URL:
+		return nautilus_drag_default_drop_action_for_url (context);
+		
 	case NAUTILUS_ICON_DND_URI_LIST :
-	case NAUTILUS_ICON_DND_URL :
 		return context->suggested_action;
+
+	case NAUTILUS_ICON_DND_TEXT:
+		return GDK_ACTION_COPY;
+
+	case NAUTILUS_ICON_DND_KEYWORD:
+
+		if (!path) {
+			return 0;
+		}
+
+		return GDK_ACTION_COPY;
 	}
 
 	return 0;
@@ -355,7 +411,9 @@ drag_motion_callback (GtkWidget *widget,
 {
 	NautilusTreeViewDragDest *dest;
 	GtkTreePath *path;
-	GtkTreePath *drop_path;
+	GtkTreePath *drop_path, *old_drop_path;
+	GtkTreeModel *model;
+	GtkTreeIter drop_iter;
 	GtkTreeViewDropPosition pos;
 	guint action;
 
@@ -372,10 +430,27 @@ drag_motion_callback (GtkWidget *widget,
 	
 	action = get_drop_action (dest, context, drop_path);
 	
+	gtk_tree_view_get_drag_dest_row (GTK_TREE_VIEW (widget), &old_drop_path,
+					 NULL);
+	
 	if (action) {
 		set_drag_dest_row (dest, drop_path);
+		model = gtk_tree_view_get_model (GTK_TREE_VIEW (widget));
+		if (drop_path == NULL || (old_drop_path != NULL &&
+		    gtk_tree_path_compare (old_drop_path, drop_path) != 0)) {
+			remove_expand_timeout (dest);
+		}
+		if (dest->details->expand_id == 0 && drop_path != NULL) {
+			gtk_tree_model_get_iter (model, &drop_iter, drop_path);
+			if (gtk_tree_model_iter_has_child (model, &drop_iter)) {
+				dest->details->expand_id = g_timeout_add (HOVER_EXPAND_TIMEOUT,
+									  expand_timeout,
+									  dest->details->tree_view);
+			}
+		}
 	} else {
 		clear_drag_dest_row (dest);
+		remove_expand_timeout (dest);
 	}
 	
 	if (path) {
@@ -386,11 +461,15 @@ drag_motion_callback (GtkWidget *widget,
 		gtk_tree_path_free (drop_path);
 	}
 	
+	if (old_drop_path) {
+		gtk_tree_path_free (old_drop_path);
+	}
+	
 	if (dest->details->scroll_id == 0) {
 		dest->details->scroll_id = 
-			gtk_timeout_add (150, 
-					 scroll_timeout, 
-					 dest->details->tree_view);
+			g_timeout_add (150, 
+				       scroll_timeout, 
+				       dest->details->tree_view);
 	}
 
 	gdk_drag_status (context, action, time);
@@ -413,6 +492,7 @@ drag_leave_callback (GtkWidget *widget,
 	free_drag_data (dest);
 
 	remove_scroll_timeout (dest);
+	remove_expand_timeout (dest);
 }
 
 static void
@@ -441,7 +521,8 @@ receive_uris (NautilusTreeViewDragDest *dest,
 		} else {
 			action = GDK_ACTION_MOVE | GDK_ACTION_COPY | GDK_ACTION_LINK;
 		}
-		context->action = nautilus_drag_drop_action_ask (action);
+		context->action = nautilus_drag_drop_action_ask
+			(GTK_WIDGET (dest->details->tree_view), action);
 	}
 
 	/* We only want to copy external uris */
@@ -500,17 +581,85 @@ receive_dropped_uri_list (NautilusTreeViewDragDest *dest,
 			  GdkDragContext *context,
 			  int x, int y)
 {
-	GList *source_uris;
-	
 	if (!dest->details->drag_data) {
 		return;
 	}
 
-	source_uris = nautilus_icon_dnd_uri_list_extract_uris ((char*)dest->details->drag_data->data);
-	
-	receive_uris (dest, context, source_uris, x, y);
-	
-	nautilus_icon_dnd_uri_list_free_strings (source_uris);
+	g_signal_emit (dest, signals[HANDLE_URI_LIST], 0,
+		       (char*)dest->details->drag_data->data,
+		       context->action,
+		       x, y);
+}
+
+static void
+receive_dropped_text (NautilusTreeViewDragDest *dest,
+		      GdkDragContext *context,
+		      int x, int y)
+{
+	char *text;
+
+	if (!dest->details->drag_data) {
+		return;
+	}
+
+	text = gtk_selection_data_get_text (dest->details->drag_data);
+	g_signal_emit (dest, signals[HANDLE_TEXT], 0,
+		       (char *) text,
+		       context->action,
+		       x, y);
+	g_free (text);
+}
+
+
+static void
+receive_dropped_url (NautilusTreeViewDragDest *dest,
+		     GdkDragContext *context,
+		     int x, int y)
+{
+	if (!dest->details->drag_data) {
+		return;
+	}
+
+	g_signal_emit (dest, signals[HANDLE_URL], 0,
+		       (char*)dest->details->drag_data->data,
+		       context->action,
+		       x, y);
+}
+
+static void
+receive_dropped_keyword (NautilusTreeViewDragDest *dest,
+			 GdkDragContext *context,
+			 int x, int y)
+{
+	char *drop_target_uri;
+	GtkTreePath *path, *drop_path;
+	NautilusFile *drop_target_file;
+
+	gtk_tree_view_get_dest_row_at_pos (dest->details->tree_view, x, y, 
+					   &path, NULL);
+
+	drop_path = get_drop_path (dest, path);
+
+	drop_target_uri = get_drop_target (dest, drop_path);
+
+	drop_target_file = nautilus_file_get (drop_target_uri);
+
+	if (drop_target_file != NULL) {
+		nautilus_drag_file_receive_dropped_keyword (drop_target_file,
+							    (char *) dest->details->drag_data->data);
+		nautilus_file_unref (drop_target_file);
+	}
+
+
+	if (path) {
+		gtk_tree_path_free (path);
+	}
+
+	if (drop_path) {
+		gtk_tree_path_free (drop_path);
+	}
+
+	g_free (drop_target_uri);
 }
 
 static gboolean
@@ -546,11 +695,22 @@ drag_data_received_callback (GtkWidget *widget,
 			receive_dropped_icons (dest, context, x, y);
 			success = TRUE;
 			break;
-		case NAUTILUS_ICON_DND_URI_LIST :
 		case NAUTILUS_ICON_DND_URL :
+			receive_dropped_url (dest, context, x, y);
+			success = TRUE;
+			break;
+		case NAUTILUS_ICON_DND_URI_LIST :
 			receive_dropped_uri_list (dest, context, x, y);
 			success = TRUE;
 			break;
+		case NAUTILUS_ICON_DND_TEXT:
+			receive_dropped_text (dest, context, x, y);
+			success = TRUE;
+			break;
+		case NAUTILUS_ICON_DND_KEYWORD:
+			receive_dropped_keyword (dest, context, x, y);
+			success = TRUE;
+			break;
 		}
 
 		dest->details->drop_occurred = FALSE;
@@ -582,11 +742,45 @@ drag_drop_callback (GtkWidget *widget,
 
 	get_drag_data (dest, context, time);
 	remove_scroll_timeout (dest);
+	remove_expand_timeout (dest);
 	clear_drag_dest_row (dest);
 	
 	return TRUE;
 }
 
+static void
+tree_view_weak_notify (gpointer user_data,
+		       GObject *object)
+{
+	NautilusTreeViewDragDest *dest;
+
+	dest = NAUTILUS_TREE_VIEW_DRAG_DEST (user_data);
+	
+	remove_scroll_timeout (dest);
+	remove_expand_timeout (dest);
+
+	dest->details->tree_view = NULL;
+}
+
+static void
+nautilus_tree_view_drag_dest_dispose (GObject *object)
+{
+	NautilusTreeViewDragDest *dest;
+	
+	dest = NAUTILUS_TREE_VIEW_DRAG_DEST (object);
+
+	if (dest->details->tree_view) {
+		g_object_weak_unref (G_OBJECT (dest->details->tree_view),
+				     tree_view_weak_notify,
+				     dest);
+	}
+	
+	remove_scroll_timeout (dest);
+	remove_expand_timeout (dest);
+
+	EEL_CALL_PARENT (G_OBJECT_CLASS, dispose, (object));
+}
+
 static void
 nautilus_tree_view_drag_dest_finalize (GObject *object)
 {
@@ -614,6 +808,7 @@ nautilus_tree_view_drag_dest_class_init (NautilusTreeViewDragDestClass *class)
 
 	gobject_class = G_OBJECT_CLASS (class);
 	
+	gobject_class->dispose = nautilus_tree_view_drag_dest_dispose;
 	gobject_class->finalize = nautilus_tree_view_drag_dest_finalize;
 
 	signals[GET_ROOT_URI] = 
@@ -643,28 +838,75 @@ nautilus_tree_view_drag_dest_class_init (NautilusTreeViewDragDestClass *class)
 					       move_copy_items),
 			      NULL, NULL,
 			      
-			      nautilus_marshal_VOID__POINTER_STRING_UINT_INT_INT,
+			      nautilus_marshal_VOID__POINTER_STRING_ENUM_INT_INT,
 			      G_TYPE_NONE, 5,
 			      G_TYPE_POINTER,
 			      G_TYPE_STRING,
-			      G_TYPE_UINT,
+			      GDK_TYPE_DRAG_ACTION,
+			      G_TYPE_INT,
+			      G_TYPE_INT);
+	signals[HANDLE_URL] =
+		g_signal_new ("handle_url",
+			      G_TYPE_FROM_CLASS (class),
+			      G_SIGNAL_RUN_LAST,
+			      G_STRUCT_OFFSET (NautilusTreeViewDragDestClass, 
+					       handle_url),
+			      NULL, NULL,
+			      nautilus_marshal_VOID__STRING_ENUM_INT_INT,
+			      G_TYPE_NONE, 4,
+			      G_TYPE_STRING,
+			      GDK_TYPE_DRAG_ACTION,
+			      G_TYPE_INT,
+			      G_TYPE_INT);
+	signals[HANDLE_URI_LIST] =
+		g_signal_new ("handle_uri_list",
+			      G_TYPE_FROM_CLASS (class),
+			      G_SIGNAL_RUN_LAST,
+			      G_STRUCT_OFFSET (NautilusTreeViewDragDestClass, 
+					       handle_uri_list),
+			      NULL, NULL,
+			      nautilus_marshal_VOID__STRING_ENUM_INT_INT,
+			      G_TYPE_NONE, 4,
+			      G_TYPE_STRING,
+			      GDK_TYPE_DRAG_ACTION,
+			      G_TYPE_INT,
+			      G_TYPE_INT);
+	signals[HANDLE_TEXT] =
+		g_signal_new ("handle_text",
+			      G_TYPE_FROM_CLASS (class),
+			      G_SIGNAL_RUN_LAST,
+			      G_STRUCT_OFFSET (NautilusTreeViewDragDestClass, 
+					       handle_text),
+			      NULL, NULL,
+			      nautilus_marshal_VOID__STRING_ENUM_INT_INT,
+			      G_TYPE_NONE, 4,
+			      G_TYPE_STRING,
+			      GDK_TYPE_DRAG_ACTION,
 			      G_TYPE_INT,
 			      G_TYPE_INT);
 }
 
+
+
 NautilusTreeViewDragDest *
 nautilus_tree_view_drag_dest_new (GtkTreeView *tree_view)
 {
 	NautilusTreeViewDragDest *dest;
+	GtkTargetList *targets;
 	
 	dest = g_object_new (NAUTILUS_TYPE_TREE_VIEW_DRAG_DEST, NULL);
 
 	dest->details->tree_view = tree_view;
+	g_object_weak_ref (G_OBJECT (dest->details->tree_view),
+			   tree_view_weak_notify, dest);
 	
 	gtk_drag_dest_set (GTK_WIDGET (tree_view),
 			   0, drag_types, G_N_ELEMENTS (drag_types),
-			   GDK_ACTION_MOVE | GDK_ACTION_COPY | GDK_ACTION_LINK | GDK_ACTION_ASK);	
-	
+			   GDK_ACTION_MOVE | GDK_ACTION_COPY | GDK_ACTION_LINK | GDK_ACTION_ASK);
+
+	targets = gtk_drag_dest_get_target_list (GTK_WIDGET (tree_view));
+	gtk_target_list_add_text_targets (targets, NAUTILUS_ICON_DND_TEXT);
+
 	g_signal_connect_object (tree_view,
 				 "drag_motion",
 				 G_CALLBACK (drag_motion_callback),
