@@ -33,6 +33,7 @@
 #define NAUTILUS_ACTION_UP_ACCEL "UpAccel"
 #define NAUTILUS_ACTION_UP_ACCEL "UpAccel"
 #define NAUTILUS_ACTION_FORWARD "Forward"
+#define NAUTILUS_ACTION_SHOW_HIDE_TOOLBAR "Show Hide Toolbar"
 #define NAUTILUS_ACTION_SHOW_HIDE_SIDEBAR "Show Hide Sidebar"
 #define NAUTILUS_ACTION_SHOW_HIDE_STATUSBAR "Show Hide Statusbar"
 #define NAUTILUS_ACTION_SHOW_HIDE_LOCATION_BAR "Show Hide Location Bar"
@@ -40,5 +41,6 @@
 #define NAUTILUS_ACTION_ZOOM_IN "Zoom In"
 #define NAUTILUS_ACTION_ZOOM_OUT "Zoom Out"
 #define NAUTILUS_ACTION_ZOOM_NORMAL "Zoom Normal"
+#define NAUTILUS_ACTION_CLOSE "Close"
 
 #endif /* NAUTILUS_ACTIONS_H */
