@@ -82,13 +82,13 @@ enum {
 	NAUTILUS_DND_NTARGETS
 };
 
-static GtkTargetEntry drag_types [] = {
+static const GtkTargetEntry drag_types [] = {
 	{ NAUTILUS_DND_URI_LIST_TYPE,   0, NAUTILUS_DND_URI_LIST },
 	{ NAUTILUS_DND_TEXT_PLAIN_TYPE, 0, NAUTILUS_DND_TEXT_PLAIN },
 	{ NAUTILUS_DND_URL_TYPE,        0, NAUTILUS_DND_URL }
 };
 
-static GtkTargetEntry drop_types [] = {
+static const GtkTargetEntry drop_types [] = {
 	{ NAUTILUS_DND_URI_LIST_TYPE,   0, NAUTILUS_DND_URI_LIST },
 	{ NAUTILUS_DND_TEXT_PLAIN_TYPE, 0, NAUTILUS_DND_TEXT_PLAIN },
 	{ NAUTILUS_DND_URL_TYPE,        0, NAUTILUS_DND_URL }
@@ -258,6 +258,33 @@ style_set_handler (GtkWidget *widget, GtkStyle *previous_style)
 	gtk_widget_set_size_request (widget, width, -1);
 }
 
+static gboolean
+label_button_pressed_callback (GtkWidget             *widget,
+			       GdkEventButton        *event)
+{
+	NautilusNavigationWindow *window;
+	NautilusView             *view;
+	GtkWidget                *label;
+
+	if (event->button != 3) {
+		return FALSE;
+	}
+
+	window = nautilus_location_bar_get_window (widget->parent);
+	view = NAUTILUS_WINDOW (window)->content_view;
+	label = GTK_BIN (widget)->child;
+	/* only pop-up if the URI in the entry matches the displayed location */
+	if (view == NULL ||
+	    strcmp (gtk_label_get_text (GTK_LABEL (label)), LOCATION_LABEL)) {
+		return FALSE;
+	}
+
+	nautilus_view_pop_up_location_context_menu (view, event);
+
+	return FALSE;
+}
+
+
 static int
 get_editable_number_of_chars (GtkEditable *editable)
 {
@@ -302,6 +329,15 @@ real_activate (NautilusNavigationBar *navigation_bar)
 	nautilus_entry_select_all (bar->details->entry);
 }
 
+static void
+real_cancel (NautilusNavigationBar *navigation_bar)
+{
+	char *last_location;
+
+	last_location = NAUTILUS_LOCATION_BAR (navigation_bar)->details->last_location;
+	nautilus_navigation_bar_set_location (navigation_bar, last_location);
+}
+
 static void
 finalize (GObject *object)
 {
@@ -357,6 +393,7 @@ nautilus_location_bar_class_init (NautilusLocationBarClass *class)
 	navigation_bar_class = NAUTILUS_NAVIGATION_BAR_CLASS (class);
 
 	navigation_bar_class->activate = real_activate;
+	navigation_bar_class->cancel = real_cancel;
 	navigation_bar_class->get_location = nautilus_location_bar_get_location;
 	navigation_bar_class->set_location = nautilus_location_bar_set_location;
 }
@@ -402,11 +439,16 @@ nautilus_location_bar_init (NautilusLocationBar *bar)
 
 	gtk_container_add (GTK_CONTAINER (bar), hbox);
 
+
+	/* Label context menu */
+	g_signal_connect (event_box, "button-press-event",
+			  G_CALLBACK (label_button_pressed_callback), NULL);
+
 	/* Drag source */
 	gtk_drag_source_set (GTK_WIDGET (event_box), 
 			     GDK_BUTTON1_MASK | GDK_BUTTON3_MASK,
 			     drag_types, G_N_ELEMENTS (drag_types),
-			     GDK_ACTION_LINK);
+			     GDK_ACTION_COPY | GDK_ACTION_LINK);
 	g_signal_connect_object (event_box, "drag_data_get",
 				 G_CALLBACK (drag_data_get_callback), bar, 0);
 
@@ -471,8 +513,11 @@ nautilus_location_bar_set_location (NautilusNavigationBar *navigation_bar,
 	
 	/* remember the original location for later comparison */
 	
-	g_free (bar->details->last_location);
-	bar->details->last_location = g_strdup (location);
+	if (bar->details->last_location != location) {
+		g_free (bar->details->last_location);
+		bar->details->last_location = g_strdup (location);
+	}
+
 	nautilus_location_bar_update_label (bar);
 }
 
@@ -484,7 +529,7 @@ nautilus_location_bar_set_location (NautilusNavigationBar *navigation_bar,
  * @bar: A NautilusLocationBar.
  *
  * returns a newly allocated "string" containing the mangled
- * (by eel_make_uri_from_input) text that the user typed in...maybe a URI 
+ * (by gnome_vfs_make_uri_from_input) text that the user typed in...maybe a URI 
  * but not guaranteed.
  *
  **/
@@ -497,7 +542,7 @@ nautilus_location_bar_get_location (NautilusNavigationBar *navigation_bar)
 	bar = NAUTILUS_LOCATION_BAR (navigation_bar);
 	
 	user_location = gtk_editable_get_chars (GTK_EDITABLE (bar->details->entry), 0, -1);
-	best_uri = eel_make_uri_from_input (user_location);
+	best_uri = gnome_vfs_make_uri_from_input (user_location);
 	g_free (user_location);
 	return best_uri;
 }
@@ -515,9 +560,9 @@ nautilus_location_bar_update_label (NautilusLocationBar *bar)
 	char *current_location;
 	
 	current_text = gtk_entry_get_text (GTK_ENTRY (bar->details->entry));
-	current_location = eel_make_uri_from_input (current_text);
+	current_location = gnome_vfs_make_uri_from_input (current_text);
 	
-	if (eel_uris_match (bar->details->last_location, current_location)) {
+	if (gnome_vfs_uris_match (bar->details->last_location, current_location)) {
 		gtk_label_set_text (GTK_LABEL (bar->details->label), LOCATION_LABEL);
 	} else {		 
 		gtk_label_set_text (GTK_LABEL (bar->details->label), GO_TO_LABEL);
