@@ -44,6 +44,7 @@
 #include <gtk/gtkmain.h>
 #include <gtk/gtkiconfactory.h>
 #include <gtk/gtksignal.h>
+#include <gdk/gdk.h>
 #include <libgnome/gnome-i18n.h>
 #include <libgnome/gnome-init.h>
 #include <libgnomeui/gnome-ui-init.h>
@@ -216,7 +217,7 @@ slowly_and_stupidly_obtain_timestamp (Display *xdisplay)
 				       0,
 				       CopyFromParent,
 				       CopyFromParent,
-				       CopyFromParent,
+				       (Visual *)CopyFromParent,
 				       CWOverrideRedirect | CWEventMask,
 				       &attrs);
 		
