@@ -36,6 +36,7 @@
 #include "nautilus-property-browser.h"
 #include "nautilus-signaller.h"
 #include "nautilus-window-manage-views.h"
+#include "nautilus-window-bookmarks.h"
 #include "nautilus-window-private.h"
 #include "nautilus-desktop-window.h"
 #include <eel/eel-debug.h>
@@ -49,6 +50,7 @@
 #include <libxml/parser.h>
 #include <gtk/gtkmain.h>
 #include <gtk/gtkaboutdialog.h>
+#include <gtk/gtkenums.h>
 #include <libgnome/gnome-help.h>
 #include <libgnome/gnome-i18n.h>
 #include <libgnome/gnome-util.h>
@@ -138,6 +140,7 @@ void
 nautilus_menus_append_bookmark_to_menu (NautilusWindow *window, 
 					NautilusBookmark *bookmark, 
 					const char *parent_path,
+					const char *parent_id,
 					guint index_in_parent,
 					GtkActionGroup *action_group,
 					guint merge_id,
@@ -166,9 +169,9 @@ nautilus_menus_append_bookmark_to_menu (NautilusWindow *window,
 	g_free (truncated_name);
 
 	/* Create menu item with pixbuf */
-	pixbuf = nautilus_bookmark_get_pixbuf (bookmark, NAUTILUS_ICON_SIZE_FOR_MENUS, FALSE);
+	pixbuf = nautilus_bookmark_get_pixbuf (bookmark, GTK_ICON_SIZE_MENU);
 
-	action_name = g_strdup_printf ("bookmark_%d", index_in_parent);
+	action_name = g_strdup_printf ("bookmark_%s_%d", parent_id, index_in_parent);
 
 	action = gtk_action_new (action_name,
 				 display_name,
@@ -215,9 +218,11 @@ static void
 action_connect_to_server_callback (GtkAction *action, 
 				   gpointer user_data)
 {
+	NautilusWindow *window = NAUTILUS_WINDOW (user_data);
 	GtkWidget *dialog;
 	
-	dialog = nautilus_connect_server_dialog_new (NAUTILUS_WINDOW (user_data));
+	dialog = nautilus_connect_server_dialog_new (window, nautilus_window_get_location (window));
+
 	gtk_widget_show (dialog);
 }
 
@@ -420,13 +425,13 @@ action_about_nautilus_callback (GtkAction *action,
 	char *logo_path;
 	GdkPixbuf *logo;
 
-	license_trans = g_strconcat (_(license[0]), "\n\n", _(license[1]), "\n\n",
-				     _(license[2]), "\n\n", NULL);
-
 	if (about == NULL) {
  
 		about = gtk_about_dialog_new ();
 
+		license_trans = g_strconcat (_(license[0]), "\n\n", _(license[1]), "\n\n",
+					     _(license[2]), "\n\n", NULL);
+
 		/* Translators should localize the following string
 		 * which will be displayed at the bottom of the about
 		 * box to give credit to the translator(s).
@@ -456,8 +461,11 @@ action_about_nautilus_callback (GtkAction *action,
 		
 		if (logo != NULL) {
 			g_object_unref (logo);
-		}
-		
+		}		
+
+		g_free (license_trans);
+		g_free (logo_path);
+
 		gtk_window_set_transient_for (GTK_WINDOW (about), GTK_WINDOW (user_data));
 
 		eel_add_weak_pointer (&about);
@@ -586,7 +594,7 @@ connect_proxy_cb (GtkUIManager *manager,
 	
 }
 
-static GtkActionEntry main_entries[] = {
+static const GtkActionEntry main_entries[] = {
   { "File", NULL, N_("_File") },               /* name, stock id, label */
   { "Edit", NULL, N_("_Edit") },               /* name, stock id, label */
   { "View", NULL, N_("_View") },               /* name, stock id, label */
@@ -632,12 +640,24 @@ static GtkActionEntry main_entries[] = {
     N_("Zoom _In"), "<control>plus",           /* label, accelerator */
     N_("Show the contents in more detail"),                                      /* tooltip */ 
     G_CALLBACK (action_zoom_in_callback) },
+  { "ZoomInAccel", NULL,                        /* name, stock id */
+    "ZoomInAccel", "<control>equal",           /* label, accelerator */
+    NULL,                                      /* tooltip */ 
+    G_CALLBACK (action_zoom_in_callback) },
+  { "ZoomInAccel2", NULL,                        /* name, stock id */
+    "ZoomInAccel2", "<control>KP_Add",           /* label, accelerator */
+    NULL,                                      /* tooltip */ 
+    G_CALLBACK (action_zoom_in_callback) },
   { "Zoom Out", GTK_STOCK_ZOOM_OUT,                        /* name, stock id */
     N_("Zoom _Out"), "<control>minus",           /* label, accelerator */
     N_("Show the contents in less detail"),                                      /* tooltip */ 
     G_CALLBACK (action_zoom_out_callback) },
+  { "ZoomOutAccel", NULL,                        /* name, stock id */
+    "ZoomOutAccel", "<control>KP_Subtract",           /* label, accelerator */
+    NULL,                                      /* tooltip */ 
+    G_CALLBACK (action_zoom_out_callback) },
   { "Zoom Normal", GTK_STOCK_ZOOM_100,                        /* name, stock id */
-    N_("Normal Si_ze"), NULL,           /* label, accelerator */
+    N_("Normal Si_ze"), "<control>0",           /* label, accelerator */
     N_("Show the contents at the normal size"),                                      /* tooltip */ 
     G_CALLBACK (action_zoom_normal_callback) },
   { "Connect to Server", NULL,                        /* name, stock id */
@@ -710,6 +730,8 @@ nautilus_window_initialize_menus (NautilusWindow *window)
 		action = gtk_action_group_get_action (action_group, NAUTILUS_ACTION_GO_TO_BURN_CD);
 		gtk_action_set_visible (action, FALSE);
 	}
+
+	nautilus_window_initialize_bookmarks_menu (window);
 }
 
 static GList *
@@ -801,3 +823,5 @@ nautilus_window_load_extension_menus (NautilusWindow *window)
 
 	g_list_free (items);
 }
+
+
